# dp_stm_map - Distributed and Persistent Software Transaction Map
# Copyright (C) 2013  Dragan Milic
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

require 'json'

module DpStmMap

  class JsonMessage

    def self.deserialize string
      # return YAML::load(string)

      json=JSON.parse(string)
      type=json.delete('type')
      cls=class_from_string(type)
      obj=cls.new
      obj.from_hash! json
      obj
    end

    def self.class_from_string(str)
      str.split('::').inject(Object) do |mod, class_name|
        mod.const_get(class_name)
      end
    end

    def serialize
        # return YAML::dump(self)
        hash = {}
        self.instance_variables.each do |var|
            hash[var] = self.instance_variable_get var
        end
        hash['type']=self.class.name
        hash.to_json
    end

    def from_hash! hash
        hash.each do |var, val|
            self.instance_variable_set var, val
        end
    end

  end



  class ClientHelloMessage < JsonMessage

    def initialize last_transaction_sequence=nil
      @last_transaction_sequence=last_transaction_sequence
    end

    attr_accessor :last_transaction_sequence

  end

  class ServerHelloMessage < JsonMessage
  end

  class ClientTransactionOutcomeMessage < JsonMessage
    def initialize transaction_id
      @transaction_id=transaction_id
    end
    attr_accessor :transaction_id
  end

  class ClientTransactionSuccessfulMessage < ClientTransactionOutcomeMessage

    def initialize transaction_id=nil, transaction_sequence=nil
      super transaction_id
      @transaction_sequence=transaction_sequence
    end

    attr_accessor :transaction_sequence

  end


  class ClientTransactionFailedMessage < ClientTransactionOutcomeMessage

    def initialize transaction_id=nil, reason=nil
      super transaction_id
      @reason=reason
    end

    attr_accessor :reason
  end

  class ClientTransactionMessage < JsonMessage
    def initialize transaction_id=nil, transitions=nil, new_content=nil
      @transaction_id=transaction_id
      @transitions=transitions
      @new_content=new_content
    end

    attr_accessor :transaction_id
    attr_accessor :transitions
    attr_accessor :new_content
  end

  class TransactionMessage < JsonMessage
    def initialize transaction_sequence=nil, transitions=nil, new_content=nil, delete_content=nil
      @transaction_sequence=transaction_sequence
      @transitions=transitions
      @new_content=new_content
      @delete_content=delete_content
    end

    attr_accessor :transaction_sequence
    attr_accessor :transitions
    attr_accessor :new_content
    attr_accessor :delete_content

  end

end
