# dp_stm_map - Distributed and Persistent Software Transaction Map
# Copyright (C) 2013  Dragan Milic
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

require 'yaml'

module DpStmMap

  class ObjectTransactionWrapper
    def initialize tx
      @tx=tx
    end

    def []= domain, key, value
      @tx[domain_key(domain,key)]=value == nil ? nil : YAML::dump(value)
    end

    def [] domain, key
      raw=@tx[domain_key(domain,key)]
      if (raw == nil)
        nil
      else
        YAML::load(raw)
      end
    end

    def has_key? domain, key
      @tx.has_key?(domain_key(domain,key))
    end

    def domain_key domain, key
      "%s:%s" % [domain.to_s, key.to_s]
    end

  end

  class ObjectStore

    def initialize stm_map
      @stm_map=stm_map
    end


    def on_atomic
      @stm_map.on_atomic do |changes|
        yield changes_to_object_changes changes
      end
    end

    def atomic timeout=nil
      @stm_map.atomic(timeout) do |stx|
        yield ObjectTransactionWrapper.new(stx)
      end
    end

    def atomic_read
      @stm_map.atomic_read do |stx|
        yield ObjectTransactionWrapper.new(stx)
      end
    end

    def validate_atomic
      @stm_map.validate_atomic do |changes|
        yield changes_to_object_changes changes
      end
    end

    private


    def changes_to_object_changes changes
      changes.inject({}) {|h, (k,v)| h[string_to_domain_key(k)]=[deserialize_object(v[0]),deserialize_object(v[1])]; h}
    end


    def string_to_domain_key str
      parts=str.split(/:/)
      [parts[0].to_sym, parts[1]]
    end

    def deserialize_object str
      str == nil ? nil : YAML::load(str)
    end

  end




end
